; se2 = A short program that is wrapped in scan records
; It is read at boot time by the BIOS and it modifies the default startup sequence
;
; BIOS gives control of the processor to this program (see Scan Record 2) so it
; has the responsibility to run the IDLE_TASK
;
; I locate this program using a linker script.
; Note that it is located at an odd boundary so that .start will be on an even boundary
; I will put it in high RAM so that I can see it after GDB has loaded (comfort factor!)
; The code is transient, it is only executed once and can therefore be overwritten
;
; Define some constants that make this program easier to read
.equ    ScanSignature,	0xC3B6
.equ    LoadCommand,	0
.equ    JumpCommand,	4
.equ    SUSB_INIT_INT,	113
.equ    IDLER_INT,	71

.section .init
; First define the code that needs to be loaded
; It will be prefixed with a Scan Header
        .short  ScanSignature
        .short  Length+2
        .byte   LoadCommand
        .short  _start
.global _start
_start:
; Adjust the default wait states for XRAM to 7
		mov r0, [0xc03a]
		or r0, 7
		mov [0xc03a], r0
; Initialize HPI
        call 0xe7c6
; Initialize SIE2 for the GDB debugger.  Use the Cypress default configuration
        mov r2, 2           ; Choose SIE2
        mov r0, 0           ; Full speed
        int SUSB_INIT_INT   ; Let the BIOS initialize SIE2
; I now need to give control back to BIOS
; I don't RETurn since BIOS will initialize SIE1, which I don't want
; So I pass control back via restarting the IDLE_TASK
		mov r15, 0x400		; Reset the stack
        sti                 ; First time interrupts are enabled
        int IDLER_INT       ; This will not return

.equ    Length, .-_start
;
; Now define a scan record that will tranfer control to my program
        .short  ScanSignature
        .short  2
        .byte   JumpCommand
        .short  _start
; Scan will not return so I need not have an EndOfScanRecords
; But use one anyway
        .short  0
