/* File: app.c */


/* Copyright 2003, Cypress Semiconductor Corporation
 *
 * This software is owned by Cypress Semiconductor Corporation (Cypress) and
 * is protected by United States copyright laws and international treaty
 * provisions. Cypress hereby grants to Licensee a personal, non-exclusive,
 * non-transferable license to copy, use, modify, create derivative works of,
 * and compile the Cypress Source Code and derivative works for the sole
 * purpose of creating custom software in support of Licensee product
 * ("Licensee Product") to be used only in conjunction with a Cypress
 * integrated circuit. Any reproduction, modification, translation,
 * compilation, or representation of this software except as specified above
 * is prohibited without the express written permission of Cypress.
 * 
 * Disclaimer: Cypress makes no warranty of any kind, express or implied, with
 * regard to this material, including, but not limited to, the implied
 * warranties of merchantability and fitness for a particular purpose. Cypress
 * reserves the right to make changes without further notice to the materials
 * described herein. Cypress does not assume any liability arising out of the
 * application or use of any product or circuit described herein. Cypress
 * products described herein are not authorized for use as components in
 * life-support devices.
 * 
 * This software is protected by and subject to worldwide patent coverage,
 * including U.S. and foreign patents. Use may be limited by and subject to
 * the Cypress Software License Agreement.
 */


/* Included files. */

#include "fwx.h"
#include "board.h"
#include "utils.h"
#include "app.h"
#include "sie1.h"
#include "host.h"
#include "debug.h"
#include "string.h"




 /* Application data. */

#ifdef FWX_INCLUDE_APP_INIT
/* This function will be called after fwxinit()
 * but prior to any invocations of app_task().
 */
void app_init(void)
{

    void (*hpi_init)(void);

    /* Disable the UART debugging capabilities of the chip. */
    
#ifndef DEBUG
    disable_serial_debug();
#endif

    /* TODO: Fill in your application's init code here. */

    /* Enable HPI Mode (BIOS function) */
    hpi_init = (void*) 0xE7C6;
    (*hpi_init)();
}
#endif /* FWX_INCLUDE_APP_INIT */



#ifdef FWX_INCLUDE_APP_TASK

/* This function will be called repeatedly by the main function.
 * mode will be one of SYSTEM_MODE_STOP, SYSTEM_MODE_HOST, or
 * SYSTEM_MODE_SLAVE.  (Actually, app_task probably won't be
 * called at all if the system mode is stop.)
 */

void app_task( FWX_SYSTEM_MODE mode[2] )
{
    switch( mode[SIE1] ) /* ??? TODO */
    {
    case SYSTEM_MODE_SLAVE:
        break;
        
    case SYSTEM_MODE_HOST:
        break;
        
    case SYSTEM_MODE_STOP:
    default:
        break;
    }
}

#endif /* FWX_INCLUDE_APP_TASK */


#ifdef FWX_APP_PARSE_DESCRIPTOR    /* FWX_APP_PARSE_DESCRIPTOR */

uint16 app_parse_descriptors( USB_DEVICE *dev, URB *urb )
{
    USB_CONFIG_DESCRIPTOR    *pCfg;
    USB_INTERFACE_DESCRIPTOR *pIntf;
    uint16                    length;
    uint8                    *cur_ptr;
    uint8                    *end_ptr;
    uint8                     descr_type;
    bool                      keyboard_found = FALSE;
    
    /* Get the config and interface desc to start the parse, 
     * to support searching for a boot keyboard. 
     */
    
    pCfg  = urb->transfer_buffer;
    
    pIntf = (USB_INTERFACE_DESCRIPTOR *) ((uint8 *)urb->transfer_buffer + pCfg->bLength);
    
    length = min( urb->actual_length, pCfg->wTotalLength );
    
    cur_ptr = (uint8 *) pCfg;
    
    end_ptr = cur_ptr + length;

    while( cur_ptr < end_ptr )
    {
        descr_type = *(cur_ptr + 1);
        
        /* Check the type of this descriptor. */
        
        switch( descr_type )
        {
        case USB_CONFIG_DESCRIPTOR_TYPE:
            {
                USB_CONFIG_DESCRIPTOR   
                       *cp = (USB_CONFIG_DESCRIPTOR *) cur_ptr;
                dev->configuration = cp->bConfigurationValue;
            }
            break;
 
        case USB_INTERFACE_DESCRIPTOR_TYPE:
            {
            USB_INTERFACE_DESCRIPTOR *pIntf = (USB_INTERFACE_DESCRIPTOR *) cur_ptr;
            
                if ( pIntf->bInterfaceClass == USB_HID_CLASS )   
                {
                    if ( pIntf->bInterfaceProtocol == USB_HID_PROTOCOL_KEYBOARD )
                    {
                        keyboard_found = TRUE;

                        /* interface will be 0 for no subclass and 1 for boot protocol,
                         * assumes only 2 HID keyboard interfaces
                         */
                        if ( pIntf->bInterfaceSubClass == USB_BOOT_INTERFACE_SUBCLASS )
                        {
                            dev->hid.boot_kb_found = TRUE;
                            
                            /* Save the interface  descriptor. */
                            memcpy( &dev->inf_descr, pIntf, pIntf->bLength );
                        }
                    }
                }
            }
            break;
        case USB_ENDPOINT_DESCRIPTOR_TYPE:
            {
                USB_ENDPOINT_DESCRIPTOR *ep;
                 
                ep = (USB_ENDPOINT_DESCRIPTOR *) cur_ptr;
                
                dev->endpoint_max_pkt[ ep->bEndpointAddress & 0xf ] = ep->wMaxPacketSize;
                
                /* interface will be 0 for no subclass and 1 for boot protocol,
                 * assumes only 2 HID keyboard interfaces
                 */
                 
                if( dev->hid.boot_kb_found == TRUE )   
                {
                    /* HID device found */
                    
                    if( ep->bmAttributes == USB_INTERRUPT_TRANSFER_TYPE )
                    {
                        /* interrupt in found */
                        
                        dev->hid.interval = ep->bInterval;
                        dev->hid.hid_int_ep = ep->bEndpointAddress & 0xf;
                        
                        /* found the boot keyboard, force the loop to terminate, otherwise
                         * the following non boot descriptor will overwrite the interval 
                         * and ep number
                         */
                        cur_ptr = end_ptr;
                    }
                }
            }
            break;

        default:
            break;
        }

        /* Increment the cur_ptr by the length of this structure.
         * The structure length is always the first byte.
         */
         
        cur_ptr += *cur_ptr;
        
    }   /* while( cur_ptr < end_ptr ) */

    
    if( keyboard_found )
    {
        if( dev->hid.boot_kb_found )
        {
            /* boot keyboard found */
            return SUCCESS;  
        }
        else
        {
            /* non boot keyboard found */
            return ERROR;    
        }
    }
    else
    {
        /* a non keyboard HID device was found */
        return SUCCESS;
    }
}


#endif /* FWX_APP_PARSE_DESCRIPTOR */



#ifdef FWX_INCLUDE_BUTTON_CALLBACKS





/* Will be called once for each button pressed.
 * Button clearing is handled at a lower level.
 */
void app_button_handler( BUTTON button )
{
    if( TRUE )
    {
        /* sie1_notify_button_change( button ); */

        switch( button )
        {
        case BTN_UP:
            cpld_inc_ssd();
            break;
        case BTN_LEFT:
            /* Put the device on the bus. */
            susb_init( SIE2, USB_LOW_SPEED );
            break;
        case BTN_RIGHT:
            cpld_clr_led( HOST_LED ); 
            cpld_clr_led( ERROR_LED ); 
            cpld_clr_led( SLAVE_LED ); 
            cpld_clr_led( SESSION_ACTIVE_LED ); 
            cpld_clear_ssd();
            /* Put the device on the bus. */
            susb_init( SIE2, USB_FULL_SPEED );
            break;
        case BTN_DOWN:
            cpld_dec_ssd();
            break;
        case BTN_CENTER:
            /* TODO: handle a press of the "CENTER" button. */
            cpld_clr_led( ERROR_LED ); 
            cpld_clear_ssd();
            break;
        default:
            /* Do nothing. */
            break;
        }
    }
    else if( fwx_get_system_mode(SIE1) == SYSTEM_MODE_SBC )
    {
    }
    else
    {
        SYS_ERROR("unhandled");
    }
}
#endif /* FWX_INCLUDE_BUTTON_CALLBACKS */





/* End of file: app.c */

