/* File: demodrvr.c */


/* Copyright 2003, Cypress Semiconductor Corporation
 *
 * This software is owned by Cypress Semiconductor Corporation (Cypress) and
 * is protected by United States copyright laws and international treaty
 * provisions. Cypress hereby grants to Licensee a personal, non-exclusive,
 * non-transferable license to copy, use, modify, create derivative works of,
 * and compile the Cypress Source Code and derivative works for the sole
 * purpose of creating custom software in support of Licensee product
 * ("Licensee Product") to be used only in conjunction with a Cypress
 * integrated circuit. Any reproduction, modification, translation,
 * compilation, or representation of this software except as specified above
 * is prohibited without the express written permission of Cypress.
 * 
 * Disclaimer: Cypress makes no warranty of any kind, express or implied, with
 * regard to this material, including, but not limited to, the implied
 * warranties of merchantability and fitness for a particular purpose. Cypress
 * reserves the right to make changes without further notice to the materials
 * described herein. Cypress does not assume any liability arising out of the
 * application or use of any product or circuit described herein. Cypress
 * products described herein are not authorized for use as components in
 * life-support devices.
 * 
 * This software is protected by and subject to worldwide patent coverage,
 * including U.S. and foreign patents. Use may be limited by and subject to
 * the Cypress Software License Agreement.
 */


/* Included files. */

#include <string.h>

#include "fwx.h"


#ifdef FWX_INCLUDE_HOST_USB_SUPPORT

#include "demodrvr.h"
#include "tpl.h"
#include "usb.h"
#include "debug.h"
#include "urb.h"
#include "td.h"
#include "app.h"
#include "board.h"
#include "logger.h"
#include "ptrlist.h"
#include "sie2.h"





#if 0
    #define DE3DRVRLOG(x,y) de3drvrlog( uint16 v1, uint16 v2 )

    static void de3drvrlog( uint16 v1, uint16 v2 )
    {
        bool    logger_in_isr = !global_ints_enabled();
    
        if( !logger_in_isr )
        {
            __asm( "cli" );
        }
    
        /* Write to a hardware trace address. */
    
        WRITE_REGISTER( HW_TRACE_ADDRESS, v1 );
    
        WRITE_REGISTER( HW_TRACE_ADDRESS, v2 );

        if( !logger_in_isr )
        {
            __asm( "sti" );
        }
    }

#else

    #define DE3DRVRLOG(x,y)

#endif










#define DE3_DEBUG  1

/* Macro definitions. */

#undef  DE3DRVR_INCLUDE_SUBMIT_URB

#define MAX_ACTIVE_URBS                         20
#define MAX_PENDED_BULK_IN_URBS                  8
#define HID_BUFFER_SIZE                          8
#define NUM_HID_BUFFERS                          1



#ifdef DEBUG
    #define ASSERT_VALID_URB(urb)                 \
            if( urb == 0 )                        \
            {                                     \
                SYS_ERROR( "not enough urbs" );   \
                return ERROR;                     \
            }
            
    #define ASSERT_VALID_SUBMIT_URB( urb)         \
            if( td_submit_URB(urb) == ERROR )     \
            {                                     \
                release_URB(urb);                 \
                SYS_ERROR( "submit failed" );     \
                return ERROR;                     \
            }
#else
    #define ASSERT_VALID_URB(urb)                 \
            if( urb == 0 )                        \
            {                                     \
                return ERROR;                     \
            }

    #define ASSERT_VALID_SUBMIT_URB( urb)         \
            if( td_submit_URB(urb) == ERROR )     \
            {                                     \
                release_URB(urb);                 \
                return ERROR;                     \
            }
#endif



/* Type definitions. */

typedef union CNVRT
{
    struct
    {
        uint8   lo;
        uint8   hi;
    } byte;
    uint16  word;
} CNVRT;


/* Data. */

DECLARE_PTRLIST( active_list, 6 );

/* Function declarations. */

/* Driver interface routines. */

static uint16 de3drvr_start( USB_DEVICE *dev );
static uint16 de3drvr_stop( USB_DEVICE *dev );
static void   de3drvr_run( USB_DEVICE *dev );
static void   hid_forward_done( void );

/* URB completion functions. */

static void interrupt_in_complete( URB *urb );


/* Helper functions. */

static URB   *alloc_interrupt_in_urb( USB_DEVICE *dev );
static uint16 resubmit_urb( URB *urb );

#ifdef DE3DRVR_INCLUDE_SUBMIT_URB
static uint16 submit_urb( URB *urb );
#endif /* DE1DRVR_INCLUDE_SUBMIT_URB */

/* interface function from sie2.c */

       void   get_HID_report( void *, uint16 length );

/* Global / Static Data. */

static uint16       usage_count = 0;

/* HID data to be sent to the Host PC  */

static uint8       hid_buffer[HID_BUFFER_SIZE]  
    ATTR_USB_XFER_BUF_SECTION __attribute__ ((aligned (2))) = { 0 };
    
static uint16      hid_buffer_busy = FALSE;
static bool        hid_buffer_dirty = FALSE;

static USBTXRXINFO hid_info ATTR_USB_XFER_BUF_SECTION __attribute__ ((aligned (2))) =
       { 0, hid_buffer, HID_BUFFER_SIZE, (PFNINTHANDLER) hid_forward_done };



CLASS_DRIVER    de3_vendor_driver =
{
    0x00,           /* uint8   class;       */
    0,              /* uint8   subclass;    */
    0x03,           /* uint8   HID if_class;    */
    0x01,           /* uint8   KB  if_subclass; */
    0x00,           /* uint8   Boot protocol;    */
    0x04B4,         /* uint16  vendor_ID;   */
    0xDE03,         /* uint16  product_ID;  */
    de3drvr_start,  /* uint16 (*start)( USB_DEVICE *dev ); */
    de3drvr_stop,   /* uint16 (*stop)(void);  */
    de3drvr_run     /* void (*run)(void);  */
};




/* DE3 Endpoints
 *  +---------------------+---------------+-----------------+
 *  |    Control in       |               |   Control /     |
 *  +---------------------+  Endpoint 0   | Vendor Cmd /    |
 *  |    Control out      |               |    Status       |
 *  +---------------------+---------------+-----------------+
 *  |   Interrupt in      |  Endpoint x   |    HID Boot     |
 *  +---------------------+---------------+-----------------+
 *  |   Interrupt in      |  Endpoint x   |    HID Non Boot |
 *  +---------------------+---------------+-----------------+
 */



/* Driver functionality. This is the driver for the boot Keyboard */

static uint16 de3drvr_start( USB_DEVICE *dev )
{
    uint16  result = ERROR;
    
    /* TODO:  We are only allowing one instance of this driver to exist.
     *        It is an error if we ever try to load more than one.
     */
    
    DBGLOG( TRC_de3drvr_start, usage_count );
     
    if( usage_count == 0 )
    {
        URB     *urb;
        
        ++usage_count;
 
        ptrlist_clear( &active_list );
        
        
        /* TODO: Complete any driver initialization that needs to
         * be performed.
         */
 
        /* cpld_set_led( HOST_LED ); */
        
        /* Submit a repeating URB on the interrupt in (EP0) pipe. 
         */
        
        urb = alloc_interrupt_in_urb( dev );
        
        ASSERT_VALID_URB(urb);
        
        ASSERT_VALID_SUBMIT_URB( urb );
        
        ptrlist_add( &active_list, urb );
        
        result = SUCCESS;
    }
    else
    {
        DBGLOG( TRC_de3drvr_start_err, 0xee01 );
        SYS_ERROR( "too many DE3 drivers" );
    }

    DBGLOG( TRC_de3drvr_start_end, usage_count );
    
    return result;
}




static uint16 de3drvr_stop( USB_DEVICE *dev )
{
    uint16  result = SUCCESS;
    URB     *urb;
    
    DBGLOG( TRC_de3drvr_stop, usage_count );
 
    if( usage_count == 1 )
    {
        DBGLOG( TRC_de3drvr_stop_2, dev );
        
        /* cpld_clr_led( HOST_LED ); */
	/* cpld_clr_led( SLAVE_LED ); */ 
        /* cpld_clr_led( ERROR_LED ); */ 
        /* cpld_clr_led( SESSION_ACTIVE_LED ); */ 
        
        /*  Clean up any activities that this driver had going. */
               
        while( ( urb = (URB *) ptrlist_remove_first( &active_list ) ) != 0 )
        {
            /* Remove the URB from the TD processor. */
            
            td_clear_URB( urb );
            
            /* Deallocate the URB. */
            
            release_URB( urb );
        }
        
        --usage_count;
    }
    else
    {
        DBGLOG( TRC_de3drvr_stop_err, 0xee01 );
    }
    
    DBGLOG( TRC_de3drvr_stop_end, usage_count );
                            
    return result;
}


/* This is the driver's idle function. */

static void de3drvr_run( USB_DEVICE *dev )
{
    static uint16 debug_count = 0;
    
    /* Make sure that we are loaded. */
    
    if( usage_count > 0 )
    {
        /* TODO */
        debug_count++;
        
        
        if( hid_buffer_dirty == TRUE )
        {
            /* Submit this buffer to Lyberty for it to transmit to the host. */

            hid_buffer_busy = TRUE;
            
            /* susb_send( SIE2, pHID_dev->hid.hid_int_ep, &hid_info ); */
            
            hid_buffer_dirty = FALSE;
        }
    }
}





/* clear the HID structures */

void clear_HID( void )
{
    hid_buffer_busy = FALSE;
    hid_buffer_dirty = FALSE;
    
    DE3DRVRLOG( TRC_clear_HID, 0 );
}




/* Send the HID data received from the keyboard to the PC Host */

void forward_HID_report( uint8 *pBuffer, uint16 length )
{
    if( hid_buffer_dirty == FALSE && hid_buffer_busy == FALSE )
    {
        hid_buffer_dirty = TRUE;
        
        hid_info.buffer = hid_buffer;  /* BIOS modifies the pointer */
        
        memcpy( hid_info.buffer , pBuffer, length );
    
        hid_info.length    = length;
        hid_info.done_func = hid_forward_done;
    }
    else
    {
        DBGLOG( TRC_forward_HID_report, 0xEE01 );
    }
}



/* HID endpoint callback function*/

static void hid_forward_done(void)
{
    hid_buffer_busy = FALSE;

    hid_buffer_dirty = FALSE;
    
    
    DE3DRVRLOG( TRC_de3_hid_forward_done, hid_buffer_busy );

    DBGLOG( TRC_de3_hid_forward_done, hid_buffer_busy );
}


/* This function is called when we have received data on the
 * interrupt in pipe on SIE1 from the keyboard.
 */
 
static void interrupt_in_complete( URB *urb )
{

    uint8 modifier = 0;
    uint8 keypress = 16;
    uint16 temp;

    if( urb->status == SUCCESS )
    {
    
        /* ********* Process this DE3 HID report here . *********** 
         *
         *          Forward the data to the PC Host, through SIE2                          
         *                              
         */
        
        modifier = *( (uint8 *)urb->transfer_buffer );
        keypress = *( (uint8 *)urb->transfer_buffer + 2 );
	temp = (modifier << 8) | keypress;
        WRITE_REGISTER(0xC0C6, temp);
        
        if ( keypress != 0 )
        {
#ifdef DE3_DEBUG
            /* debug, crack the report packet to find a character to print on the SSD,
             * only printing 0 to f on the SSD
             */
            if( (keypress > 0x00) && (keypress < 0x0a) ) keypress += 6; /* a-f */
            if( (keypress > 0x1d) && (keypress < 0x27) ) keypress -= 0x1d; /* 1-9 */
            if( (keypress >= 0x27) ) keypress = 0; /* 0 or larger value */
            
            /* write_cpld( SSD_WRITE, ssd_mask[value] ); */
            /* cpld_set_ssd( keypress ); */
#endif     
        }

	clear_HID();

    }
    else
    {
        /* We need to resubmit this URB to the TD processor.
         * Keep in mind that it is still on our active list.
         */

        resubmit_urb( urb );
        DBGLOG( TRC_de3_intr_complete_5, 0XEE03 );
    }
    return;
}





static URB *alloc_interrupt_in_urb( USB_DEVICE *dev )
{
    URB *urb;
    
    urb = alloc_URB( FALSE, dev->endpoint_max_pkt[dev->hid.hid_int_ep] );

    if( urb != 0 )
    {                                       
        urb->dev = dev;
        urb->dir = TD_CTRL_DIR_IN;
        urb->usb_dev_addr = dev->address;
        urb->endpoint = dev->hid.hid_int_ep;
        
        /* 0 = full or 1 = low speed */
        urb->speed = dev->speed;            

        /* end point type: iso, intr, control, bulk */
        urb->type = USB_INTERRUPT_TRANSFER_TYPE;         

        /* data buffer length */
        urb->buffer_length = dev->endpoint_max_pkt[dev->hid.hid_int_ep];

        /* polling interval (irq only) */
        urb->interval = dev->hid.interval;

        /* callback for URB completion */
        urb->callback = interrupt_in_complete;     
    }
    else
    {
        SYS_ERROR( "not enough URBs" );
    }
    
    return urb;
}
 


 



static uint16 resubmit_urb( URB *urb )
{
    uint16  result;
    
    urb->status = SUCCESS;
    urb->error_count = 0;
    
    result = td_submit_URB( urb );
    
    if( result != SUCCESS )
    {
        /* Remove from our active list. */
                    
        ptrlist_remove( &active_list, urb );
        
        /* Deallocate the URB. */
        
        release_URB( urb );

        SYS_ERROR( "Resubmit failed" );
    }
    
    return result;
}




#endif /* FWX_INCLUDE_HOST_USB_SUPPORT */




/* End of file: de3drvr.c */

