/* File: app.c */

/*
 * Copyright 2003, Cypress Semiconductor Corporation
 *
 * This software is owned by Cypress Semiconductor Corporation (Cypress) and
 * is protected by United States copyright laws and international treaty
 * provisions.  Therefore, unless otherwise specified in a separate license
 * agreement, you must treat this software like any other copyrighted
 * material.  Reproduction, modification, translation, compilation, or
 * representation of this software in any other form (e.g., paper, magnetic,
 * optical, silicon, etc.) is prohibited without the express written
 * permission of Cypress.
 *
 * Disclaimer: Cypress makes no warranty of any kind, express or implied, with
 * regard to this material, including, but not limited to, the implied
 * warranties of merchantability and fitness for a particular purpose.
 * Cypress reserves the right to make changes without further notice to the
 * materials described herein. Cypress does not assume any liability arising
 * out of the application or use of any product or circuit described herein.
 * Cypress' products described herein are not authorized for use as
 * components in life-support devices.
 *
 * This software is protected by and subject to worldwide patent coverage,
 * including U.S. and foreign patents. Use may be limited by and subject to
 * the Cypress Software License Agreement.
 */



/* Included files. */

#include "fwx.h"
#include "board.h"
#include "utils.h"
#include "app.h"
#include "sie1.h"
#include "host.h"
#include "debug.h"
#include "string.h"
#include "prnclass.h"




/* Macros. */




/* Application data. */

#ifdef FWX_INCLUDE_APP_INIT
/* This function will be called after fwxinit()
 * but prior to any invocations of app_task().
 */
void app_init(void)
{
    void (*hpi_init)(void);

    /* Disable the UART debugging capabilities of the chip. */
    
    disable_serial_debug();

    /* Enable HPI Mode (BIOS function) */
    hpi_init = (void*) 0xE7C6;
    (*hpi_init)();
}
#endif /* FWX_INCLUDE_APP_INIT */



#ifdef FWX_INCLUDE_APP_TASK

/* This function will be called repeatedly by the main function.
 * mode will be one of SYSTEM_MODE_STOP, SYSTEM_MODE_HOST, or
 * SYSTEM_MODE_SLAVE.  (Actually, app_task probably won't be
 * called at all if the system mode is stop.)
 */

void app_task( FWX_SYSTEM_MODE mode[2] )
{
    switch( mode[SIE1] ) /* ??? TODO */
    {
    case SYSTEM_MODE_SLAVE:
        break;
        
    case SYSTEM_MODE_HOST:
        break;
        
    case SYSTEM_MODE_STOP:
    default:
        break;
    }
}

#endif /* FWX_INCLUDE_APP_TASK */





#ifdef FWX_INCLUDE_BUTTON_CALLBACKS

/* Will be called once for each button pressed.
 * Button clearing is handled at a lower level.
 */
 
void app_button_handler( BUTTON button )
{
    switch( button )
    {
    case BTN_UP:
        break;

    case BTN_LEFT:
        break;

    case BTN_RIGHT:
        break;

    case BTN_DOWN:
        WRITE_REGISTER( HSS_DATA_REG, 'R' );
        break;

#ifdef CPLD_INCLUDE_CENTER_BUTTON        
    case BTN_CENTER:
        tpl_ioctl( PRNCLASS_DEV_ID, PRNCLASS_IOCTL_SEND_TEST_PAGE, 0, 0 );
        break;
#endif /* CPLD_INCLUDE_CENTER_BUTTON */

    default:
        /* Do nothing. */
        break;
    }
}
#endif /* FWX_INCLUDE_BUTTON_CALLBACKS */





#ifdef FWX_APP_PARSE_DESCRIPTOR

uint16 app_parse_descriptors( USB_DEVICE *dev, URB *urb )
{
    USB_CONFIG_DESCRIPTOR   *cfg;
    uint16                  length;
    uint8                   *cur_ptr;
    uint8                   *end_ptr;
    uint8                   descr_type;
    bool                    inf_descr_used = FALSE;
    
    /* Look for an OTG descriptor. */
    
    cfg = urb->transfer_buffer;
    
    length = min( urb->actual_length, cfg->wTotalLength );
    
    cur_ptr = (uint8 *) cfg;
    
    end_ptr = cur_ptr + length;

    while( cur_ptr < end_ptr )
    {
        descr_type = *(cur_ptr + 1);
        
        /* Check the type of this descriptor. */
        
        switch( descr_type )
        {
        case USB_OTG_DESCRIPTOR_TYPE:
            {
                USB_OTG_DESCRIPTOR  *otg = (USB_OTG_DESCRIPTOR *) cur_ptr;
                dev->otg_attributes = otg->bmAttributes;
            }
            break;

        case USB_ENDPOINT_DESCRIPTOR_TYPE:
            {
                USB_ENDPOINT_DESCRIPTOR *ep = (USB_ENDPOINT_DESCRIPTOR *) cur_ptr;
                dev->endpoint_max_pkt[ ep->bEndpointAddress & 0xf ] = ep->wMaxPacketSize;
            }
            break;

        case USB_CONFIG_DESCRIPTOR_TYPE:
            {
                USB_CONFIG_DESCRIPTOR   *cp = (USB_CONFIG_DESCRIPTOR *) cur_ptr;
                dev->configuration = cp->bConfigurationValue;
            }
            break;
            
        case USB_INTERFACE_DESCRIPTOR_TYPE:
            {
                USB_INTERFACE_DESCRIPTOR *ip = (USB_INTERFACE_DESCRIPTOR *) cur_ptr;
                
                /* The first one wins. */
                
                if( inf_descr_used == FALSE && prnclass_use_this_intf_descr( ip ) == TRUE )
                {
                    USB_ENDPOINT_DESCRIPTOR *ep = (USB_ENDPOINT_DESCRIPTOR *) (ip + 1);
                    
                    memcpy( &dev->inf_descr, ip, sizeof(USB_INTERFACE_DESCRIPTOR) );
                    
                    /* Let the driver know about our chosen interface. */
        
                    prnclass_specify_interface( ip, ep );
                    
                    inf_descr_used = TRUE;
                }
            }
            break;
            
        default:
            break;
        }

        /* Increment the cur_ptr by the length of this structure.
         * The structure length is always the first byte.
         */
         
        cur_ptr += *cur_ptr;
    }
    
    return SUCCESS;
}

#endif /* FWX_APP_PARSE_DESCRIPTOR */



/* End of file: app.c */

